#!/bin/sh

if [ ! -x /opt/lsb-tet3-lite/bin/tcc ]
then
	printf "tcc not found, aborting\n"
	exit 1
fi

TET_ROOT=`pwd`
TET_PATH=/opt/lsb-tet3-lite
MANPATH=$MANPATH:$TET_PATH/man
PATH=$TET_PATH/bin:$TET_ROOT/xtest/bin:$PATH
export PATH MANPATH TET_ROOT

function config_param()
{

DEFAULT=`grep "^$1=" xtest/tetexec.cfg | cut -d= -f2`
printf "$2 ("

if [ -n "$DEFAULT" ]; 
then
  printf "$DEFAULT"
fi

printf ")? "
read cmd

if [ -z "$cmd" ]; then
  cmd=$DEFAULT
fi

# Write to temporary file
echo "$1=$cmd" >> /tmp/newparam.$$

}

function configure()
{
    config_param VSW_OPER "Name of person running tests"
    config_param VSW_ORG  "Organisation"
    config_param VSW_SYS "Test System"
    config_param XT_DISPLAYHOST "Hostname of client for X tests"
    config_param XT_DISPLAY "DISPLAY of the Xvfb (ie., :1)"
    config_param XT_FONTPATH "XT_FONTPATH"
    config_param XT_FONTPATH_GOOD "XT_FONTPATH_GOOD"

    mv $TET_ROOT/xtest/tetexec.cfg $TET_ROOT/xtest/tetexec.cfg.old
    cat $TET_ROOT/xtest/tetexec.cfg.old | $TET_ROOT/xtest/bin/editcfg.sh /tmp/newparam.$$ > $TET_ROOT/xtest/tetexec.cfg
    mkfontdir $TET_ROOT/xtest/fonts

    if [ $? -ne 0 ]; then
        echo Failed to install fonts
        exit 1
    fi

}

# We need to run the tests using the X Virtual Frame Buffer xvfb(1), 
# because using graphics cards gives varying results.
#

# Check whether Xvfb can be found in current PATH settings
which Xvfb > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "Cannot find Xvfb executable binary in your current PATH setting:"
	echo "$PATH"
	echo "Please make sure you have installed the necessary Xvfb package"
	echo "(virtual frame buffer X server, whose name is something like"
	echo "xvfb, XFree86-Xvfb or xorg-x11-Xvfb) and add the path which"
        echo "includes the Xvfb executable binary to your PATH setting"
	exit 1
fi

# Configure the test suites
if [ ! -f .configured ]; then
    configure
    touch .configured
fi

XT_DISPLAY=`grep '^XT_DISPLAY=' xtest/tetexec.cfg | cut -f 2 -d=`
TEST_DISPLAY="`expr $XT_DISPLAY : '.*\(:[0-9][0-9]*\.[0-9][0-9]*\)'`" 
XT_FONTPATH=`grep '^XT_FONTPATH=' xtest/tetexec.cfg | cut -f 2 -d=`

pkill Xvfb
(Xvfb -render -screen 0 1024x768x16 $TEST_DISPLAY -ac +bs -kb -fp $XT_FONTPATH &)
echo "Starting X Virtual Frame Buffer"; sleep 20
export DISPLAY=$TEST_DISPLAY

# Test that test fonts are installed
echo "Testing the test fonts are installed .... "
xlsfonts xtfont0 | grep xtfont0 
if [ $? -ne 0 ]; then
  echo 'Could not find xtfont0. xlsfonts xtfont? should list xtfont0..xtfont6'
  echo 'Aborting tests'
  exit 1
fi

cd xtest && tcc -p -e -v DISPLAY=$TEST_DISPLAY -s link_scen xtest linkexec

pkill Xvfb

#EOF
