#########################################################################
#									#
#	SCCS:	@(#) wrptdet.awk	Rel 1.3 (7/20/92)			#
#									#
#	UniSoft Ltd., London, England
#									#
# (C) Copyright 1991 X/Open Company Limited				#
#									#
# All rights reserved.  No part of this source code may be reproduced,	#
# stored in a retrieval system, or transmitted, in any form or by any	#
# means, electronic, mechanical, photocopying, recording or otherwise,	#
# except as stated in the end-user licence agreement, without the prior	#
# permission of the copyright owners.					#
#									#
# X/Open and the 'X' symbol are trademarks of X/Open Company Limited in	#
# the UK and other countries.						#
#									#
#########################################################################

#########################################################################
#									#
# NAME:		wrptdet.awk - produce report from vSX journal		#
#									#
# PROJECT:	VSW (X/OPEN Validation Suite)				#
#									#
# AUTHOR:	David McCardle, UniSoft Ltd.				#
#									#
# MODIFICATIONS:							#
#	Geoff Clare,  7 Jan 1988:  Add VERBOSE parameter		#
#	Geoff Clare, 11 Jan 1988:  Add COLS parameter			#
#	Geoff Clare,  9 Jun 1988:  Move summary stuff to wrptsum.awk	#
#	Geoff Clare, 13 Jun 1988:  Change format of area summary	#
#	Geoff Clare, 19 Sep 1988:  Recognise %..td, %..ts		#
#	Geoff Clare, 11 May 1989:  Recognise new %..x[sx] lines		#
#	Geoff Clare, 21 July 1989: New result codes			#
#									#
#	J.A. Nave Wed Nov 28 16:39:27 GMT 1990				#
#		Modified to use new style TET journal files		#
#									#
#	Geoff Clare, 27 Feb 1991					#
#		Substantial re-write					#
#									#
#	Geoff Clare, 21 June 1991					#
#		Add -t option (truncate test info)			#
#									#
#	Geoff Clare, 6 Sept 1991					#
#		use date parameter in new format TCC start line		#
#									#	
#	Stuart Boutell, 21 November 1991				#
#		Add -p option (disable page of page numbering)		#
#									#
#	Stuart Boutell, February 1992					#
#		Migrated to VSW from VSX				#
#									#
#########################################################################

BEGIN {
	FS = "|"
	vsw_sys = vsw_org = "Unknown"
	prog = "Test"
	in_range["Section"] = "NO"
	in_range["Area"] = "NO"
	in_range["Test-Set"] = "NO"
	cover_read = "NO"
	group = "none"
	end_time = "Unknown"
	vpcnt = 0
	err_num = 1
	months = "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec "
	ncodes = 8 # number of f_code[]/f_type[] entries EXCLUDING "UNREPORTED"
	f_code[1] = "1"
	f_code[2] = "101"
	f_code[3] = "102"
	f_code[4] = "2"
	f_code[5] = "6"
	f_code[6] = "4"
	f_code[7] = "5"
	f_code[8] = "3"
	f_code[9] = "7"
	f_type["1"] = "Failed"
	f_type["101"] = "Warning"
	f_type["102"] = "Further Information Provided"
	f_type["2"] = "Unresolved"
	f_type["6"] = "Uninitiated"
	f_type["4"] = "Unsupported"
	f_type["5"] = "Untested"
	f_type["3"] = "Not In Use"
	f_type["7"] = "UNREPORTED"
	for (i = 1; i <= ncodes; i++)
	{
	    ix = f_code[i]
	    f_cnt[ix] = f_test[ix] = area_tot[ix] = sect_tot[ix] = 0
	}
	area_tot["suc"] = sect_tot["suc"] = 0
}


NR == 1 { DTE = $0; next }
NR == 2 { CMD = $0; next }
NR == 3 { LEVEL = $0; next }
NR == 4 {
	RANGE = $0
	if ( RANGE != "all" )
	{
	    rnge_num = split(RANGE, rnge, " ")
	    for (i = 1; i <= rnge_num; i++)
	    {
		if ( index(rnge[i], ":") != 0 )
		{
		    split(rnge[i], set, ":")
		    range[i",type"] = "set"
		    range[i",low"] = set[1]
		    range[i",high"] = set[2]
		}
		else
		{
		    range[i",type"] = "single"
		    range[i",low"] = range[i",high"] = rnge[i]
		}
	    }
	}
	next
}
NR == 5 { DEBUG = $0; next }
NR == 6 { HEADER = $0; next }
NR == 7 { COVER = $0; next }
NR == 8 { LINES = $0; next }
NR == 9 { COLS = 0 + $0; next }
NR == 10 { NAMES = $0; next }
NR == 11 { VERBOSE = $0; next }
NR == 12 { TRUNCATE = $0; next }
NR == 13 { PAGENUMB = $0; next }

$1 == "-1" {
	jnlfilenum++
	jnlfiles[jnlfilenum] = $2
	vpstart[jnlfilenum] = vpcnt
	next
}


{
	# because FS is "|", if any line contains a "|" in the text part it
	# will be split, so combine fields 3-NF here into "text_part" variable

	if (NF < 3)
	    text_part = ""
	else
	{
	    text_part = $3
	    for (i = 4; i <= NF+0; i++)
		text_part = text_part "|" $i
	} 
}

$1 == "-3" {
	# Test description

	if (prt_info == 1 && text_part != "")
	{
	    if (td_done == 0)
	    {
		printf "%s\n%s\tTest Description:\n", \
		    beg_str["th"], beg_str["th"]
		td_done = 1
	    }

	    print beg_str["th"] text_part
	}
	next
}
$1 != "-3" {
	td_done = 0
}

$1 == "-2" {
	# Test strategy

	if (prt_info == 1 && text_part != "")
	{
	    if (ts_done == 0)
	    {
		printf "%s\n%s\tTest Strategy:\n", \
		    beg_str["th"], beg_str["th"]
		ts_done = 1
	    }

	    print beg_str["th"] text_part
	}
	next
}
$1 != "-2" {
	ts_done = 0
}

prt_info == 1 {
	printf "%s\n%s\t%s Information:\n", \
	    beg_str["th"], beg_str["th"], prog
	printf "%s", sav_text[tpres[2]]
	printf "%s", beg_str["th"]
	for (i = offset["th"]; i <= COLS; i++)
	    printf "*"
	printf "\n"

	prt_info = 0
	for (ix in sav_text)
	{
	    sav_text[ix] = ""
	    sav_cnt[ix] = 0
	}
}

$1 == "520" {
	# TC Information
	# Since the result code is after this info in the journal,
	# we have to store the info for printing after the result.
	# Some awk's can't handle the amount of data to be stored,
	# so truncate after the number of lines specified in TRUNCATE.
	# (TRUNCATE == 0 means no truncation).

	if (in_range["Test-Set"] == "YES" && text_part != "")
	{
	    split($2, tmp, " ")
	    if (TRUNCATE+0 > 0 && sav_cnt[tmp[2]] == TRUNCATE+0)
		text_part = "TEST INFORMATION TRUNCATED BY -t OPTION"
	    if (TRUNCATE+0 <= 0 || sav_cnt[tmp[2]]++ <= TRUNCATE)
		sav_text[tmp[2]] = sav_text[tmp[2]] beg_str["th"] text_part "\n"
	}
}

$1 == "220" {
	# TP Result

        split($2, tpres, " ")
        if (tpres[3] == "0")	# report a pass
        {
            if (in_range["Section"] == "YES")
                suc_test++
	    if (in_range["Test-Set"] == "YES" && VERBOSE == "YES")
	    {
	        if (trh_prt == 0)
	        {
            	    printf "%s\n", beg_str["th"]
            	    printf "%s%s Results:\n%s\n", beg_str["th"], prog, \
                        beg_str["th"]
		    trh_prt++
	        }
	        printf "%s%s %s\tSucceeded\n", beg_str["th"], path, tpres[2]
	    }
        }
        else	# report a failure
        {
            if (in_range["Section"] == "YES")
	        f_test[tpres[3]]++
            if (in_range["Area"] == "YES")
	    {
		# Don't include NOTINUSE unless verbose mode
                if ((VERBOSE == "YES" && tpres[3] == "3") || (tpres[3] != "3"))
                {
	            if (prog == "Test")
		    {
			if (tsetdir ~ /^M/)
			    f_group[tpres[3] "," f_cnt[tpres[3]]] = \
				"M" testset " " tpres[2]
			else
			    f_group[tpres[3] "," f_cnt[tpres[3]]] = \
				testset " " tpres[2]
			f_cnt[tpres[3]]++
		    }
                }
            }
            if (in_range["Test-Set"] == "YES")
            {
		# Don't include NOTINUSE unless verbose mode
                if ((VERBOSE == "YES" && tpres[3] == "3") || (tpres[3] != "3"))
                {
	            if (trh_prt == 0)
	            {
            	        printf "%s\n", beg_str["th"]
            	        printf "%s%s Results:\n", beg_str["th"], prog
		        trh_prt++
	            }
                    printf "%s\n%s", beg_str["th"], beg_str["th"]
                    for (i = offset["th"]; i <= COLS; i++)
                        printf "*"
	            if (prog == "Test")
	            {
            	        printf "\n%s%s %s\t%s\n", \
			    beg_str["th"], path, tpres[2], f_type[tpres[3]]
	            }
	            else
            	        printf "\n%s%s\t%s\n", beg_str["th"], path, f_type[tpres[3]]
		    prt_info = 1
                }
		else
		{
		    for (ix in sav_text)
		    {
			sav_text[ix] = ""
			sav_cnt[ix] = 0
		    }
		}
            }
        }
}

$1 == "400" {
	# IC Start

        if (in_range["Section"] == "YES")
        {
            split($2, tmp, " ")
            n_tests += tmp[3]
        }
}

$1 == "130" || $1 == "320" {
	# Build End or Clean End

	split($2, tmp, " ")
	end_time = tmp[3]
}

$1 == "410" {
	# IC End - since there is no Test Case End line, take testset
	# end time as last IC End time

	split($2, tmp, " ")
	end_time = tmp[4]
}

$1 == "10" || $1 == "110" || $1 == "300" || $1 == "900" {
	# Test Case Start or Build Start or Clean Start or TCC End

        if ($1 == "900")
        {
            testset = tsetdir = area = section = ""
        }
        else
        {
            split($2, tmp, " ")
            path = tmp[2]
            n = split(path, comp, "/")
	#   m = split(comp[n], tc, ".")
	#   testset = tc[m]
	    testset = tsetdir = comp[n-1]
	    area = comp[n-2]
	    section = comp[n-3]
        }
        tc_name = section "/" area "/" tsetdir


	# Output the testset tail if it is not the first "Start" line

        if (otsetdir != "")
        {
            if (in_range["Section"] == "YES")
            {
    	        area_nmr += n_tests
                area_tot["suc"] += suc_test
	        for (ix in f_test)
                    area_tot[ix] += f_test[ix]
            }
            if (in_range["Test-Set"] == "YES")
            {
                printf "%s\n%s", beg_str["th"], beg_str["th"]
                printf "Test-Set Ended:   %s\n", end_time
                printf "%s\n%s", beg_str["th"], beg_str["th"]
                printf "Test-Set Results Summary:\n"
                printf "%s------------------------\n", beg_str["th"]
                printf "%s\n%s", beg_str["th"], beg_str["th"]
                printf "%4d  %ss Executed\n%s", n_tests, prog, beg_str["th"]
	        if (suc_test > 0)
            	    printf "%4d  %ss Succeeded\n%s", suc_test, prog, beg_str["th"]
	        u_test = n_tests - suc_test
	        for (i = 1; i <= ncodes; i++)
	        {
		    ix = f_code[i]
		    if (f_test[ix] > 0)
		    {
		        printf "%4d  %ss %s\n%s", f_test[ix], prog, f_type[ix], beg_str["th"]
		        u_test -= f_test[ix]
		    }
	        }
	        if (u_test > 0)
		    printf "%4d  %ss UNREPORTED\n%s", u_test, prog, beg_str["th"]
                printf "%s\n", beg_str["ah"]
            }
            else if (in_range["Area"] == "YES" && LEVEL == "Area")
	    {
	        u_test = n_tests - suc_test
	        for (i = 1; i <= ncodes; i++)
		    u_test -= f_test[f_code[i]]
		if (otsetdir ~ /^M/)
		    printf "%s%-11s", beg_str["ah"], "M" otestset
		else
		    printf "%s%-11s", beg_str["ah"], otestset
		if (prog == "Test")
		{
		    printf "%5d", n_tests
		    for (i = 1; i <= ncodes; i++)
			printf "%5d", f_test[f_code[i]]
		    printf "%5d\n", u_test
		}
		else
		    printf "%9d%9d%9d\n", suc_test, f_test["1"], f_test["4"]
            }
            n_tests = 0
        }

	# Output the area tail if we have a new section or area and it is
	# not the first "Start" line

	if (oarea != "" && (section != osection || area != oarea))
        {
            if (in_range["Section"] == "YES")
            {
                sect_tset += tset
	        for (ix in area_tot)
                    sect_tot[ix] += area_tot[ix]
	        sect_nmr += area_nmr;
            }
            if (in_range["Area"] == "YES")
            {
                name_len = length(area "/" tsetdir)
                if (NAMES == "YES")
                {
                    if (name_len < 8)
                        beg_str["ah"] = area "/" tsetdir"\t\t"
                    else if (name_len < 16)
                        beg_str["ah"] = area "/" tsetdir"\t"
                    else
                        beg_str["ah"] = area "/" tsetdir
                }
                else
                    beg_str["ah"] = "\t"
                if (LEVEL == "Test-Set")
	        {
	    	    if (newpage != "YES")
		        printf ".bp\n"
	            newpage = "NO"
            	    printf "%s\n%s", beg_str["ah"], beg_str["ah"]
		    title = osection "/" oarea
            	    printf "Area Name: %s\n%s", title, beg_str["ah"]
                    n_len = length(title)
            	    for (i = 1; i <= n_len + 11; i++)
                        printf "-"
		    printf "\n%s\n", beg_str["ah"]
         	    printf "%sArea Started: %s\n", beg_str["ah"], area_start
		    printf "%s\n", beg_str["ah"]
		    if (prog == "Test")
		    {
			printf \
"%sTest Set        Failed      FIP   Uninit'ed  Untested  Unreported\n", beg_str["ah"]
			printf \
"%s          Executed  Warnings Unreslved Unsupported NotInUse\n", beg_str["ah"]
		    }
		    else
			printf "%sTest Set      Succeeded  Failed Unsupported\n", beg_str["ah"]
	        }

	        u_test = area_nmr - area_tot["suc"]
	        for (i = 1; i <= ncodes; i++)
		    u_test -= area_tot[f_code[i]]

	        printf "%s%-11s", beg_str["ah"], "TOTAL"
	        if (prog == "Test")
	        {
		    printf "%5d", area_nmr
		    for (i = 1; i <= ncodes; i++)
		        printf "%5d", area_tot[f_code[i]]
		    printf "%5d\n", u_test
	        }
	        else
		    printf "%9d%9d%9d\n", area_tot["suc"], area_tot["1"], area_tot["4"]
                printf "%s\n", beg_str["ah"]
                printf "%sArea Ended:   %s\n", beg_str["ah"], end_time
                printf "%s\n", beg_str["ah"]
	        if (prog == "Test" && f_cnt["1"] + f_cnt["2"] + f_cnt["6"] + f_cnt["4"] > 0)
	        {
		    printf "%s\n%s%-18s%-18s%-18s%s\n", beg_str["ah"], \
		        beg_str["ah"], "Failed", "Unresolved", "Uninitiated", "Unsupported"
		    printf "%s\n", beg_str["ah"]
		    idx1["1"] = idx1["2"] = idx1["6"] = idx1["4"] = 0
		    while (idx1["1"] < f_cnt["1"] || idx1["2"] < f_cnt["2"] || \
		           idx1["6"] < f_cnt["6"] || idx1["4"] < f_cnt["4"])
		    {
		        # find lowest failed testset name alphabetically
		        min = "zzzzzzzz"
		        for (grp in idx1)
		        {
			    if (idx1[grp] < f_cnt[grp])
			    {
			        split(f_group[grp "," idx1[grp]], s, " ")
			        if (s[1] < min)
				    min = s[1]
			    }
		        }

		        # print failures for lowest failed testset name
		        printf "%s", beg_str["ah"]

		        split(f_group["1," idx1["1"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["1," idx1["1"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["2," idx1["2"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["2," idx1["2"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["6," idx1["6"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["6," idx1["6"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["4," idx1["4"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["4," idx1["4"]++]
		        else
			    printf "%-18s", ""
		    
		        printf "\n"
		    }
	        }
                f_grp_empty = 0
                for (i=0; i<f_cnt[3]; i++)
                {
                   if (f_group["3,"i] != "")
                   {
                       f_grp_empty++
                       break;
                   }
                }
	        if (prog == "Test" && f_cnt["101"] + f_cnt["102"] + f_cnt["5"] + f_cnt["3"] > 0 && f_grp_empty)
	        {
		    printf "%s\n%s%-18s%-18s%-18s%s\n", beg_str["ah"], \
		        beg_str["ah"], "Warnings", "FIP", "Untested", "Not In Use"
		    printf "%s\n", beg_str["ah"]
		    idx2["101"] = idx2["102"] = idx2["5"] = idx2["3"] = 0
		    while (idx2["101"] < f_cnt["101"] || idx2["102"] < f_cnt["102"] || \
		           idx2["5"] < f_cnt["5"] || idx2["3"] < f_cnt["3"])
		    {
		        # find lowest failed testset name alphabetically
		        min = "zzzzzzzz"
		        for (grp in idx2)
		        {
			    if (idx2[grp] < f_cnt[grp])
			    {
			        split(f_group[grp "," idx2[grp]], s, " ")
			        if (s[1] < min)
				    min = s[1]
			    }
		        }

		        # print failures for lowest failed testset name
		        printf "%s", beg_str["ah"]

		        split(f_group["101," idx2["101"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["101," idx2["101"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["102," idx2["102"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["102," idx2["102"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["5," idx2["5"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["5," idx2["5"]++]
		        else
			    printf "%-18s", ""

		        split(f_group["3," idx2["3"]], s, " ")
		        if (s[1] == min)
			    printf "%-18s", f_group["3," idx2["3"]++]
		        else
			    printf "%-18s", ""
		    
		        printf "\n"
		    }
	        }
            }
        }

	# Output the section tail if we have a new section and it is
	# not the first "Start" line

        if (osection != "" && section != osection)
        {
            if (in_range["Section"] == "YES")
            {
	        if (newpage != "YES")
	            printf ".bp\n"
	        newpage = "NO"
                printf "%sSection Name: %s\n", beg_str["sh"], osection
                printf "%s", beg_str["sh"]
                n_len = length(osection)
                for (i = 1; i <= n_len + 14; i++)
                    printf "-"
                printf "\n%s\n", beg_str["sh"]
                printf "%sSection Started: %s\n", beg_str["sh"], sect_start
                printf "%s\n", beg_str["sh"]
                printf "%s\n%s", beg_str["sh"], beg_str["sh"]
                printf "Section Ended:   %s\n", end_time
                printf "%s\n%s", beg_str["sh"], beg_str["sh"]
                printf "Section Results Summary:\n"
                printf "%s", beg_str["sh"]
                printf "-----------------------\n%s\n", beg_str["sh"]
                printf "%s", beg_str["sh"]
                printf "\t%d\tAreas Containing   %d  Test-Sets Completed\n", area_cnt, sect_tset
                printf "%s\n", beg_str["sh"]
                printf "%s\t%4d  %ss Executed\n", beg_str["sh"], sect_nmr, prog
	        if (sect_tot["suc"] > 0)
            	    printf "%s\t%4d  %ss Succeeded\n", beg_str["sh"], sect_tot["suc"], prog
	        u_test = sect_nmr - sect_tot["suc"]
	        for (i = 1; i <= ncodes; i++)
	        {
		    ix = f_code[i]
		    if (sect_tot[ix] > 0)
		    {
		        printf "%s\t%4d  %ss %s\n", beg_str["sh"], sect_tot[ix], prog, f_type[ix]
		        u_test -= sect_tot[ix]
		    }
	        }
	        if (u_test > 0)
		    printf "%s\t%4d  %ss UNREPORTED\n", beg_str["sh"], u_test, prog
                printf "%s\n", beg_str["sh"]
            }

        }

        if ($1 == "900")
	    tset_start = ""
        else
	{
            split($2, tmp, " ")
	    tset_start = tmp[3]
	}
	if (section != osection || area != oarea)
		area_start = tset_start
	if (section != osection)
		sect_start = tset_start
	end_time = "Unknown"

	# Determine whether we are coming in or out of range

        if (RANGE == "all")
        {
	    in_range["Section"] = "YES"
	    if (LEVEL == "Area" || LEVEL == "Test-Set")
		in_range["Area"] = "YES"
	    if (LEVEL == "Test-Set")
		in_range["Test-Set"] = "YES"
        }
        else
        {
	    # If oldname is any range[high] then level is no longer in range
            for (i = 1; i <= rnge_num; i++)
            {
                r_num = split(range[i",high"], r_pieces, "/")

		if (r_num >= 1 && osection != section && \
			osection == r_pieces[1])
		{
		    in_range["Section"] = "NO"
		    if (r_num == 1)
		    {
			if (LEVEL == "Area" || LEVEL == "Test-Set")
			    in_range["Area"] = "NO"
			if (LEVEL == "Test-Set")
			    in_range["Test-Set"] = "NO"
		    }
		}
		if (r_num >= 2 && \
			(osection != section || oarea != area) && \
			osection == r_pieces[1] && oarea == r_pieces[2])
		{
		    if (LEVEL != "Section")
			in_range["Area"] = "NO"
		    if (r_num == 2)
		    {
			if (LEVEL == "Test-Set")
			    in_range["Test-Set"] = "NO"
		    }
		}
		if (r_num == 3 && LEVEL == "Test-Set" && \
			(osection != section || oarea != area || \
			otsetdir != tsetdir) && osection == r_pieces[1] && \
			oarea == r_pieces[2] && otsetdir == r_pieces[3])
		    in_range["Test-Set"] = "NO"
		else if (r_num == 4 && LEVEL == "Test-Set" && \
			(osection != section || oarea != area || \
			otsetdir != tsetdir || otestset != testset) && \
			osection == r_pieces[1] && oarea == r_pieces[2] && \
			otsetdir == r_pieces[3] && "T." otestset == r_pieces[4])
		    in_range["Test-Set"] = "NO"
	    }

	    # If newname is any range[low] then level is now in range
            for (i = 1; i <= rnge_num; i++)
            {
                r_num = split(range[i",low"], r_pieces, "/")

		if (r_num >= 1 && osection != section && \
			section == r_pieces[1])
		{
		    in_range["Section"] = "YES"
		    if (r_num == 1)
		    {
			if (LEVEL == "Area" || LEVEL == "Test-Set")
			    in_range["Area"] = "YES"
			if (LEVEL == "Test-Set")
			    in_range["Test-Set"] = "YES"
		    }
		}
		if (r_num >= 2 && \
			(osection != section || oarea != area) && \
			section == r_pieces[1] && area == r_pieces[2])
		{
		    if (LEVEL != "Section")
			in_range["Area"] = "YES"
		    if (r_num == 2)
		    {
			if (LEVEL == "Test-Set")
			    in_range["Test-Set"] = "YES"
		    }
		}
		if (r_num == 3 && LEVEL == "Test-Set" && \
			(osection != section || oarea != area || \
			otsetdir != tsetdir) && section == r_pieces[1] && \
			area == r_pieces[2] && tsetdir == r_pieces[3])
		    in_range["Test-Set"] = "YES"
		else if (r_num == 4 && LEVEL == "Test-Set" && \
			(osection != section || oarea != area || \
			otsetdir != tsetdir || otestset != testset) && \
			section == r_pieces[1] && area == r_pieces[2] && \
			tsetdir == r_pieces[3] && "T." testset == r_pieces[4])
		    in_range["Test-Set"] = "YES"
	    }
	}

	# Reset section parameters if we have a new section

        if (section != osection && section != "")
        {
            if (in_range["Section"] == "YES")
            {
                sect++;
                area_cnt = 0
	        sect_nmr = 0
                sect_tset = 0
	        for (ix in sect_tot)
                    sect_tot[ix] = 0
                if (NAMES == "YES")
                    beg_str["sh"] = section"\t"
                else
                    beg_str["sh"] = "\t"
            }
	    if (newpage != "YES")
	    {
	        printf ".bp\n"
	        newpage = "YES"
	    }
        }

	# Reset area parameters if we have a new area

        if ((section != osection || area != oarea) && area != "")
        {
            if (in_range["Section"] == "YES")
            {
		area_cnt++
                tset = 0
	        area_nmr = 0
	        for (ix in area_tot)
		    area_tot[ix] = 0
	        for (ix in f_cnt)
		    f_cnt[ix] = 0
            }
            if (in_range["Area"] == "YES")
            {
                name_len = length(area "/" tsetdir)
                if (NAMES == "YES")
                {
                    if (name_len < 8)
                        beg_str["ah"] = area "/" tsetdir"\t\t"
                    else if (name_len < 16)
                        beg_str["ah"] = area "/" tsetdir"\t"
                    else
                        beg_str["ah"] = area "/" tsetdir
                }
                else
                    beg_str["ah"] = "\t"
	        if (newpage != "YES")
	        {
		    printf ".bp\n"
		    newpage = "YES"
	        }
                if (LEVEL != "Test-Set")
	        {
		    newpage = "NO"
            	    printf "%s\n%s", beg_str["ah"], beg_str["ah"]
                    title = section "/" area
            	    printf "Area Name: %s\n%s", title, beg_str["ah"]
                    n_len = length(title)
            	    for (i = 1; i <= n_len + 11; i++)
                        printf "-"
		    printf "\n%s\n", beg_str["ah"]
         	    printf "%sArea Started: %s\n", beg_str["ah"], area_start
           	    printf "%s\n", beg_str["ah"]
		    if (prog == "Test")
		    {
			    printf \
"%sTest Set        Failed      FIP   Uninit'ed  Untested  Unreported\n", beg_str["ah"]
			    printf \
"%s          Executed  Warnings Unreslved Unsupported NotInUse\n", beg_str["ah"]
		    }
		    else
			printf "%sTest Set      Succeeded  Failed Unsupported\n", beg_str["ah"]
           	    printf "%s\n", beg_str["ah"]
	        }
            }
        }

	# Output testset header if we have a new testset

        if ($1 != "900")
        {
            if (in_range["Section"] == "YES")
            {
		tset++
                suc_test = 0
	        for (ix in f_test)
		    f_test[ix] = 0
            }
            if (in_range["Test-Set"] == "YES")
            {
                if (NAMES == "YES")
                {
                    name_len = length(path)
                    if (name_len < 8)
                    beg_str["th"] = path"\t\t\t"
                    else if (name_len < 16)
                        beg_str["th"] = path"\t\t"
                    else if (name_len < 24)
                        beg_str["th"] = path"\t"
                    else
                        beg_str["th"] = path
                    offset["th"] = 33
                }
                else
                {
                    beg_str["th"] = "\t"
                    offset["th"] = 9
                }
	        if (newpage == "NO")
		    printf ".bp\n"
	        newpage = "TSET"
                printf "%s\n%s", beg_str["th"], beg_str["th"]
                printf "Test-Set Name: %s\n%s", path, beg_str["th"]
	        n_len = length(path)
                for (i = 1; i <= n_len + 15; i++)
                    printf "-"
                printf "\n%s\n%sTest-Set Results:\n", beg_str["th"], beg_str["th"]
                printf "%s----------------\n%s\n", beg_str["th"], beg_str["th"]
                printf "%sTest-Set Started: %s\n", beg_str["th"], tset_start
            }
	    trh_prt = 0
        }

	osection = section
	oarea = area
	otsetdir = tsetdir
	otestset = testset
}

$1 == "50" || $1 == "90" || $1 == "510" {
	# Error messages

	if (in_range["Section"] == "YES")
	{
	    if ($1 == "50")
		err_text = sprintf("*** TCC Message: %s ***", text_part)
	    else if ($1 == "90")
		err_text = sprintf("*** OPERATOR ABORT at %s ***", $2)
	    else if ($1 == "510")
		err_text = sprintf("*** TCM/API Message: %s ***", text_part)

	    if (cover_read == "YES")
		print err_text
	    else
		err_store[err_num++] = err_text
	}
}

$1 == "20" {
	# Config Start

	# Determine if build or clean mode
	if ($2 ~ /0 *$/)
	{
		prog = "Make"
		break
	}
	else if ($2 ~ /2 *$/)
	{
		prog = "Clean"
		break
	}
}

$1 == "30" {
	# Config variables

	n = index(text_part, "=")
	vp[vpcnt",1"] = substr(text_part, 1, n-1)
	vp[vpcnt",2"] = substr(text_part, n+1)
        if (vp[vpcnt",1"] == "VSW_ORG")
            vsw_org = vp[vpcnt",2"]
        else if (vp[vpcnt",1"] == "VSW_SYS")
            vsw_sys = vp[vpcnt",2"]
	vpcnt++
}

$1 == "0" {
	# TCC Start

	# Extract test date
	n = split($2, v, " ")
	if (n >= 3)
	{
	    # New format (VSW and VSX 4.1.2 on): 3rd parameter is YYYYMMDD
	    i = substr(v[3], 5, 2) # month number
	    month = substr(months, i*4-3, 4)
	    vf_date = month substr(v[3], 7, 2) ", " substr(v[3], 1, 4)
	}
	else
	{
	    # Old VSX 4.1.1 format
	    # This relies on undocumented format of text part of TCC Start line
	    n = split(text_part, v, " ")
	    vf_date = v[9] " " v[10] ", " v[12]
	}
}

$1 == "40" {
	# Configuration End

        cover_read = "YES"
	if (COVER == "NO" || prog == "Make" || prog == "Clean")
	{
		# Parameters for wrpt.page
		printf "%s\n%s\n%s\n%s\n", HEADER, LEVEL, LINES, COLS
		printf "Test Agency: %s\n", vsw_org
		printf "System Tested: %s\n", vsw_sys
		printf "%s Date:   %s\n", prog, vf_date
		printf "%s\n", PAGENUMB
	}
        for (i = 1; i < err_num; i++)
                printf "%s\n", err_store[i]
	err_num = 1
	newpage = "YES"
}

END {
	if (newpage != "YES")
	    printf ".bp\n"
        if (COVER == "YES")
        {
            printf "%s Parameters:\n", prog
	    vpstart[jnlfilenum+1] = vpcnt
	    for (file = 1; file <= jnlfilenum; file++)
	    {
		if (jnlfilenum > 1)
		    printf "\nJournal File %s:\n\n", jnlfiles[file]
		else
		    printf "\n"
		for (i = vpstart[file]; i < vpstart[file+1]; i++)
			printf "\t%-20s =\t%s\n", vp[i",1"], vp[i",2"]
	    }
	    printf "\n\nReport Command Line:\t%s\n", CMD
	    printf ".bp\n"
        }
}
