#########################################################################
#									#
#	SCCS:	@(#)common/vrpt/wrptsum.awk	Rel 1.6 (7/16/92)	#
#									#
#	UniSoft Ltd., London, England
#									#
# (C) Copyright 1991 X/Open Company Limited
#
# All rights reserved.  No part of this source code may be reproduced,	#
# stored in a retrieval system, or transmitted, in any form or by any	#
# means, electronic, mechanical, photocopying, recording or otherwise,	#
# except as stated in the end-user licence agreement, without the prior	#
# permission of the copyright owners.					#
#									#
# X/Open and the 'X' symbol are trademarks of X/Open Company Limited in	#
# the UK and other countries.						#
#									#
#########################################################################

#########################################################################
#									#
# NAME:		vrptsum.awk - produce summary report from		#
#				vtest/vmake output			#
#									#
# PROJECT:	VSW (X/OPEN Window Management Validation Suite)		#
#									#
# AUTHOR:	Geoff Clare, UniSoft Ltd.				#
#									#
# DATE:		1st June 1988						#
#									#
# MODIFICATIONS:							#
#		Geoff Clare, 21 July 1989				#
#			new 3.1 result codes				#
#			separate counts for macro areas			#
#									#
#		J.A. Nave Wed Nov 28 16:39:27 GMT 1990			#
#		Modified to use new style TET journal files		#
#									#
#		Geoff Clare, 03 Dec 1990				#
#			update for XPG4					#
#									#
#		Geoff Clare, 21 June 1991				#
#			add -t option					#
#									#
#		Geoff Clare, 19 July 1991				#
#			add FIP sign-off page				#
#									#
#		Geoff Clare, 6 Sept 1991				#
#			use date parameter in new format TCC start line	#
#									#
#		Geoff Clare, 13 Nov 1991				#
#			FIPS mode					#
#									#
#		Stuart Boutell, 20 Nov 1991				#
#			Made 'Number of ammendments' change		#
#			Add signature front page.			#
#			Add -p option.					#
#									#
#########################################################################

BEGIN {
	sectid = osection = ""
	NUMSUBS = 1
	subname[1] = "X/Open Window Management"
	numsects[1] = 1
	subsects["1,1"] = "tset"

	sectname["tset"] = "X/Open Window Management"

	EXP="exp"
	prog = "Test"
	months = "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec "
	sectnum = 0
	for (ix in subname)
		values[ix] = "NO"
	vtermok = "YES"
	incomplete = "NO"
	areatype = "F"
	FS = "|"
}

NR == 1 { DTE = $0; next }
NR == 2 { CMD = $0; next }
NR == 3 { LEVEL = $0; next }
NR == 4 { RANGE = $0; next }
NR == 5 { DEBUG = $0; next }
NR == 6 { HEADER = $0; next }
NR == 7 { COVER = $0; next }
NR == 8 { LINES = $0; next }
NR == 9 { COLS = 0 + $0; next }
NR == 10 { NAMES = $0; next }
NR == 11 { VERBOSE = $0; next }
NR == 12 { TRUNCATE = $0; next }
NR == 13 { PAGENUMB = $0; next }

# The most frequent pattern matches are at the top for speed

prog == "Make" { next }  # Don't want a summary for make/clean reports
prog == "Clean" { next } # (if just exit vrpt can't write to pipe)

$1 == "520" {
	# Test Information
	# Since the result code is after this info in the journal,
	# we have to store the info for printing after the result.
	# Some awk's can't handle the amount of data to be stored,
	# so truncate after the number of lines specified in TRUNCATE.
	# (TRUNCATE == 0 means no truncation).

	sav_ti = ""
	sav_cnt = 0
	ok = 1
	while ($1 == "520" && ok > 0)
	{
	    if ($3 != "")
	    {
		if (TRUNCATE+0 > 0 && sav_cnt == TRUNCATE+0)
		    text_part = "TEST INFORMATION TRUNCATED BY -t OPTION"
		else
		{
		    text_part = $3
		    for (i = 4; i <= NF+0; i++)
			text_part = text_part "|" $i
		}
		if (TRUNCATE+0 <= 0 || sav_cnt++ <= TRUNCATE)
		    sav_ti = sav_ti "\t" text_part "\n"
	    }
	    ok = getline
	}
	# no "next" here: must drop through after getline
}

$1 == "-3" && fip_cnt > 0 && fip_td[fip_cnt] == "" {
	# Test Description

	ok = 1
	while ($1 == "-3" && ok > 0)
	{
	    if ($3 != "")
	    {
		fip_td[fip_cnt] = fip_td[fip_cnt] "\t" $3
		for (i = 4; i <= NF+0; i++)
		    fip_td[fip_cnt] = fip_td[fip_cnt] "|" $i
		fip_td[fip_cnt] = fip_td[fip_cnt] "\n"
	    }
	    ok = getline
	}
	# no "next" here: must drop through after getline
}

$1 == "220" {
	# TP Result

	split($2, tpres, " ")
	if (tpres[3] != "7")	# don't include NORESULT
		count[sectid "," areatype ",exe"]++
	if (tpres[3] == "0")	# report a pass
		count[sectid "," areatype ",suc"]++
	else
		count[sectid "," areatype "," tpres[3]]++
	if (tpres[3] == "102")
	{
		fip_cnt++
		fip_id[fip_cnt] = tset_name " " tpres[2]
		fip_ti[fip_cnt] = sav_ti
		fip_td[fip_cnt] = ""
	}
	next
}

$1 == "0" {
	# TCC Start

	# Extract test date
	n = split($2, v, " ")
	if (n >= 3)
	{
	    # New format (VSW and VSX 4.1.2 on): 3rd parameter is YYYYMMDD
	    i = substr(v[3], 5, 2) # month number
	    month = substr(months, i*4-3, 4)
	    vf_date = month substr(v[3], 7, 2) ", " substr(v[3], 1, 4)
	    vh_date = vf_date " " v[2]
	}
	else
	{
	    # Old VSX 4.1.1 format
	    # This relies on undocumented format of text part of TCC Start line
	    n = split($3, v, " ")
	    vh_date = v[8] " " v[9] " " v[10] " " v[11] " " v[12]
	    vf_date = v[9] " " v[10] ", " v[12]
	}

	next
}

$1 == "20" {
	# Config Start

	# Determine if build or clean mode
	if ($2 ~ /0 *$/)
	{
		prog = "Make"
		break
	}
	else if ($2 ~ /2 *$/)
	{
		prog = "Clean"
		break
	}
}

$1 == "30" {
	# Config Parameter

	n = split($3, tmp, "=")
	vp[vpcnt",1"] = tmp[1]
	if (vp[vpcnt",1"] == "VSW_ORG")
	    vsw_org = tmp[2]
	if (vp[vpcnt",1"] == "VSW_SYS")
	    vsw_sys = tmp[2]
	if (vp[vpcnt",1"] == "VSW_OPER")
	    vsw_oper = tmp[2]
	vp[vpcnt++",2"] = tmp[2]
	next
}

$1 == "-1" {
	# Start new journal file

	jnlfilenum++
	jnlfiles[jnlfilenum] = $2
	if (vtermok == "NO")
		incomplete = "YES"
	vtermok = "NO"
	next
}

$1 == "10" || $1 == "900" {
        # TC Start or TCC End

	if ($1 == "900")
	{
		vtermok = "YES"
		sectid = osection
	}
	else
	{
		split($2, tcs, " ")
		tset_name = tcs[2]
		n = split(tset_name, comp, "/")
		m = split(comp[n], tc, ".")
		testset = tc[m]
		sectid = comp[n-3]
	}

	if (comp[n-1] ~ /M/)
		areatype = "M"
	else
		areatype = "F"

	if (sectid != osection || $1 == "900")
	{
		if (sectid != osection)
		{
			sectnum++
			sections[sectnum] = sectid
		}
		for (subset = 1; subset <= NUMSUBS; subset++)
		{
			for (sect = 1; sect <= numsects[subset]; sect++)
			{
				if (subsects[subset "," sect] == sectid)
					values[subset] = "YES"
			}
		}
		osection = sectid
	}
	next
}

$1 == "70" {
	# Scenario Information Line (contains expected counts)

	n = split($3, scen_info, " ")
	split(scen_info[n], exp_count, "\"")
	sector = scen_info[n-1]
	count[sector ",F," EXP] = exp_count[1]
	if (sector ~ /\.os/ && sector != "XPG2.os")
		count[sector ",M," EXP] = exp_count[1]
	next
}

END {
	if (prog == "Make" || prog == "Clean")
		exit	# no summary for make/clean reports


	# Parameters for vrpt.page
	printf "%s\n%s\n%s\n%s\n", HEADER, LEVEL, LINES, COLS
	printf "Test Agency: %s\n", vsw_org
	printf "System Tested: %s\n", vsw_sys
	printf "Test Date:   %s\n", vf_date
	printf "%s\n", PAGENUMB

	printf "\n\n\n\n\n"
	str = "X/OPEN Window Management Verification Suite " LEVEL " Summary Report"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n\n\n"
	str = "VSW Release 4.1.1"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n\n\n\n"
	printf "\n\tTest Engineer:\t%s\t\tSignature:\n", vsw_oper
	printf "\n\tJob Title:\t\t\t\tDate:\n"
	printf "\n\tOrganisation:\t%s\n", vsw_org
	printf "\n\n\n\tClient Name:\t\t\t\tSignature:\n"
	printf "\n\tJob Title:\t\t\t\tDate:\n"
	printf "\n\tOrganisation:\n"
	printf "\n\n\n\tTest Date:\t\t%s\n", vh_date
	printf "\n\tTest System:\t\t%s\n", vsw_sys
	printf ".bp\n\n\n\n\n\n"
	str = "Report Contents"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
		printf "_"
	printf "\n\n\n"
	printf "\n\t1. Operational Summary\n"
	printf "\n\t2. Conformance Summary\n"
	printf "\n\t3. FIP Results Signatures\n"
	for (sect = 1; sect <= sectnum; sect++)
	{
		sectid = sections[sect]
		name = sectname[sectid]
		if (name == "")
			name = sectid
		printf "\n\t%d. %s Section Report\n", sect+3, name
	}
	printf "\n\t%d. Test Parameters\n", sect+3
	printf ".bp\n\n\n\n\n"
	str = "Operational Summary"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
		printf "_"
	printf "\n\n\n"
	printf "\n\tTest Date:    \t\t%s\n", vh_date
	printf "\n\tTest Agency:  \t\t%s\n", vsw_org
	printf "\n\tTest Operator:\t\t%s\n\n\n\n", vsw_oper
	printf "\n\tReport Date:  \t\t%s\n", DTE
	printf "\n\tReport Level: \t\t%s\n", LEVEL
	printf "\n\tReport Range: \t\t%s\n\n\n\n", RANGE
	if (jnlfilenum == 1)
		printf "\n\tJournal File: \t\t%s\n", jnlfiles[1]
	else
	{
		printf "\n\tJournal Files:\n\n"
		for (i = 1; i <= jnlfilenum; i++)
			printf "\t\t%s\n", jnlfiles[i]
	}
	printf "\n\n\n\tSystem Configuration\n\n"
	printf "\t\tHardware Description:\n\n\n\n\n"
	printf "\t\tSoftware Description and Release:\n"
	printf ".bp\n"

	printf "CONFORMANCE Summary\n"
	printf "\n\tThis is to certify that this system when tested for\n"
	printf "\tconformance to the X/Open Window Management Specification\n"
	printf "\tachieved the results below.\n\n"
	if (incomplete == "YES" || vtermok == "NO")
	{
		printf "*****  THESE RESULTS ARE INCOMPLETE DUE TO PREMATURE END OF JOURNAL FILE  *****\n\n"
	}
	for (subset = 1; subset <= NUMSUBS; subset++)
	{
		printf "%-4s subset:", subname[subset]
		if (values[subset] == "NO")
			printf " No tests executed\n\n\n"
		else
		{
			printf "\n%s\n", \
"            |    TOTALS   Succeeded   Warnings Unresolved Unsupported NotInUse|"
			printf "%s\n", \
"Section     | Expect Actual       Failed      FIP    Uninitiated  Untested    |"
			printf "%s\n", \
"____________|_________________________________________________________________|"
			printf "%s\n", \
"            |                                                                 |"
			tot[EXP] = tot["exe"] = tot["suc"] = tot["1"] = \
				tot["101"] = tot["102"] = tot["2"] = tot["6"] = \
				tot["4"] = tot["5"] = tot["3"] = 0
			for (sect = 1; sect <= numsects[subset]; sect++)
			{
			    for (i = 1; i <= 2; i++)
			    {
			    	if (i == 1)
					areatype = "F"
				else
					areatype = "M"

				sectid = subsects[subset "," sect]
				if (count[sectid "," areatype "," EXP] == 0 && \
				    count[sectid "," areatype ",exe"] == 0)
					continue
				if (sectid ~ /\.os/ && sectid != "XPG2.os")
					name = sprintf("%-10s%s", sectid, areatype)
				else
					name = sectid

				printf "%-12s|%6d %6d %6d %5d %4d %4d %5d %5d %5d %4d %4d |\n", \
					name, \
					count[sectid "," areatype "," EXP], \
					count[sectid "," areatype ",exe"], \
					count[sectid "," areatype ",suc"], \
					count[sectid "," areatype ",1"], \
					count[sectid "," areatype ",101"], \
					count[sectid "," areatype ",102"], \
					count[sectid "," areatype ",2"], \
					count[sectid "," areatype ",6"], \
					count[sectid "," areatype ",4"], \
					count[sectid "," areatype ",5"], \
					count[sectid "," areatype ",3"]
				for (ix in tot)
					tot[ix] += count[sectid "," areatype "," ix]
			    }
			}
			printf "%-12s|%6d %6d %6d %5d %4d %4d %5d %5d %5d %4d %4d |\n", \
				"TOTAL", tot[EXP], tot["exe"], tot["suc"], \
				tot["1"], tot["101"], tot["102"], tot["2"], \
				tot["6"], tot["4"], tot["5"], tot["3"]
			printf "%s\n\n", \
"____________|_________________________________________________________________|"
		}
	}

	printf "\n\tNumber of amendments ____________\n"
	str = "Signature/Date"
	ind = (COLS - length(str)) - 6
	printf "%" ind "s", ""
	for (i = 1; i <= length(str) + 5; i++)
	    printf "_"
	printf "\n%" ind "s%s\n", "", str
	printf ".bp\n\n\n"

	str = "FIP Results Signatures"
	ind = (COLS - length(str))/2
	printf "%" ind "s%s\n", "", str
	printf "%" ind "s", ""
	for (i = 1; i <= length(str); i++)
	    printf "_"
	printf "\n"
	for (nfip = 1; nfip <= fip_cnt; nfip++)
	{
	    printf "\n\n\t%s\n\n", fip_id[nfip]
	    printf "\t\tTest Description:\n%s\n", fip_td[nfip]
	    printf "\t\tTest Information:\n%s\n", fip_ti[nfip]
	    printf "\tSignature/Date ____________________________________\n"
	}
	printf ".bp\n"
}
